@@ -6,6 +6,8 @@
 ##      section of settings.xml, so the system remains coherent       ##
 ########################################################################
 
+::Version::setSubversionId {$Id: skins.tcl 11592 2009-09-03 04:01:44Z kakaroto $}
+
 namespace eval ::skin {
 
 	variable preview_skin_change 0
@@ -64,6 +66,11 @@ namespace eval ::skin {
 		
 		::skin::setKey expand_xpad 5
 		::skin::setKey expand_ypad 0
+
+		::skin::setKey contactlist_xpad 10
+		::skin::setKey contactlist_ypad 10
+		
+		::skin::setKey mydp_hoverimage 0
 		
 	}
 
@@ -126,13 +133,90 @@ namespace eval ::skin {
 		if { [catch {set loaded_${location}($pixmap_name) [image create photo ${naming}_${pixmap_name} -file [::skin::GetSkinFile ${location} [set ${location}_names($pixmap_name)] \
 			"" [set ${location}_fblocation($pixmap_name)]] -format cximage] } res ] } {
 		 	status_log "Error while loading pixmap $res"
-		 	set loaded_${location}($pixmap_name) [image create photo ${naming}_${pixmap_name} -file [::skin::GetSkinFile pixmaps null \
-			 -format cximage]]
+		 	set loaded_${location}($pixmap_name) [image create photo ${naming}_${pixmap_name}]
 		 }
 	
 		return [set loaded_${location}($pixmap_name)]
 	}
 
+	################################################################
+	# ::skin::newFont
+	# Creates a new font based on values from the settings.xml file.
+	proc newFont {cstack cdata saved_data cattr saved_attr args} {
+		upvar $saved_data sdata
+		
+		#Check if no important fields are missing.
+		#The name field must be present.
+		if { ! [info exists sdata(${cstack}:name)] } { return 0 }
+
+		set font_name ""
+		set font_family [lindex [::config::getGlobalKey basefont] 0]
+		set font_size [lindex [::config::getGlobalKey basefont] 1]
+		set font_weight "normal"
+		set font_slant "roman"
+		set font_underline "false"
+
+		foreach field [array names sdata] {
+			# Iterate the data.
+			switch [string replace $field 0 15] {
+				name {
+					if {[string trim $sdata($field)] != "default"} {
+						set font_name [string trim $sdata($field)]
+					}
+				}
+				family {
+					if {[string trim $sdata($field)] != "default"} {
+						set font_family [string trim $sdata($field)]
+					}
+				}
+				size {
+					if {[string trim $sdata($field)] != "default"} {
+						set font_size [string trim $sdata($field)]
+					}
+				}
+				weight {
+					set font_weight [string trim $sdata($field)]
+				}
+				slant {
+					if {[string trim $sdata($field)] == "none"} {
+						set font_slant "roman"
+					} else {
+						set font_slant [string trim $sdata($field)]
+					}
+				}
+				underline {
+					set font_underline [string trim $sdata($field)]
+				}
+				default {
+					status_log "::skin::newFont => Illegal argument: [string replace $field 0 15]. Value: [string trim $sdata($field)]."
+				}
+			}
+		}
+		
+		if {[lsearch [font names] $font_name] != "-1"} {
+			font delete $font_name
+		}
+		
+		#puts "$font_name \"$font_family\" $font_size $font_weight $font_slant $font_underline"
+		font create "$font_name" -family "$font_family" -size "$font_size" -weight "$font_weight" -slant "$font_slant" -underline "$font_underline"
+		
+		# This is required by sxml.
+		return 0
+	}
+	
+	################################################################
+	# ::skin::getFont (fontname [, default_font])
+	# Looks for the font requested, and if it isn't there it returns the default font given.
+	# Arguments:
+	#  - fontname => The name of the font to load from the skins config file.
+	#  - default => The name of the font to load if the font isn't available from the skin.
+	proc getFont {fontname {default "bplainf"}} {
+		if {[lsearch [font names] $fontname] != "-1"} {
+			return $fontname
+		} else {
+			return $default
+		}	
+	}
 
 	################################################################
 	# ::skin::getNoDisplayPicture ([skin_name])
@@ -142,7 +226,7 @@ namespace eval ::skin {
 	# Arguments:
 	#  - skin_name => [NOT REQUIRED] Overrides the current skin.
 	proc getNoDisplayPicture { {skin_name ""} } {
-		variable loaded_images
+		global loaded_images
 		if { [info exists loaded_images(displaypicture_std_none)] } {
 			return displaypicture_std_none
 		}
@@ -153,16 +237,28 @@ namespace eval ::skin {
 
 	proc getDisplayPicture { email {force 0}} {
 		global HOME
-				
+
 		set picName displaypicture_std_$email
-		
+
+		#@@@@@@@@@ webMSN display picture (thanx to majinsoftware)
+		if { [::abook::getContactData $email client] == "Webmessenger" } { 
+			return [::skin::loadPixmap webmsn_dp]
+		}
 		if {[catch {image width $picName}] == 0  && $force == 0} {
 			return $picName
 		}
 
-		set filename [::abook::getContactData $email displaypicfile ""]
-		set file "[file join $HOME displaypic cache ${filename}].png"
-		if { $filename != "" && [file readable "[file join $HOME displaypic cache ${filename}].png"] } {
+		if { [::abook::getContactData $email customdp] != "" } {
+			set filename [::abook::getContactData $email customdp ""]
+			# As custom DP can also be stored outside the cache folder,
+			# customdp stores the full path to the image
+			set file $filename
+		} else {
+			set filename [::abook::getContactData $email displaypicfile ""]
+			set file "[file join $HOME displaypic cache $email ${filename}].png"
+		}
+		
+		if { $filename != "" && [file readable "$file"] } {
 			catch {image create photo $picName -file "$file" -format cximage}
 		} else {
 			# We do like that to have a better update of the image when switching from no pic to another
@@ -190,21 +286,42 @@ namespace eval ::skin {
 			#Little DP exists so we have to update it
 			::skin::getLittleDisplayPicture $email $height 1
 		}
+		set loaded_images($picName) 1
 		return $picName
 	}
+	proc getLittleDisplayPictureName { email } {
+		return "displaypicture_tny_$email"
+	}
 
-	proc getLittleDisplayPicture { email height {force 0}} {
+
+	proc getLittleDisplayPicture { email { height 20 } {force 0}} {
 		global HOME
 
-		set picName displaypicture_tny_$email
+		set picName [getLittleDisplayPictureName $email]
+
+		#@@@@@@@@@@ WebMSN Patch again by majinsoftware
+		if { [::abook::getContactData $email client] == "Webmessenger" } { 
+			set picName [::skin::loadPixmap webmsn_dp]
+			::picture::ResizeWithRatio $picName $height $height
+			return $picName
+		}
 		
-		if {[catch {image width $picName}] == 0  && $force == 0} {
+		set imagetype ""
+
+		if {[catch {image type $picName} imagetype] == 0 && $imagetype != "" && $force == 0} {
 			return $picName
 		}
 
-		set filename [::abook::getContactData $email displaypicfile ""]
-		set file "[file join $HOME displaypic cache ${filename}].png"
-
+		if { [::abook::getContactData $email customdp] != "" } {
+			set filename [::abook::getContactData $email customdp ""]
+			# As custom DP can also be stored outside the cache folder,
+			# customdp stores the full path to the image
+			set file $filename
+		} else {
+			set filename [::abook::getContactData $email displaypicfile ""]
+			set file "[file join $HOME displaypic cache $email ${filename}].png"
+		}
+		
 		if { $filename != "" && [file readable $file] } {
 			catch {image create photo $picName -file "$file" -format cximage}
 		} else {
@@ -213,17 +330,12 @@ namespace eval ::skin {
 			set file [::skin::GetSkinFile displaypic nopic.gif]
 		}
 		
-		if {[catch {image width $picName} res]} {
-			#status_log "Error while loading $picName: $res"
+		if {[catch {image type $picName} imagetype] != 0 || $imagetype == ""} {
+			status_log "Error while loading $picName: $imagetype"
 			image create photo $picName
 			$picName copy [::skin::getNoDisplayPicture]
 			set file [::skin::GetSkinFile displaypic nopic.gif]
 		}
-		
-		if { [catch {::picture::GetPictureSize $file} cursize] } {
-			status_log "Error opening $file: $cursize\n"
-			return ""
-		}
 
 		set animated [::picture::IsAnimated $file]
 
@@ -306,15 +418,19 @@ namespace eval ::skin {
 	# This function is for the Snack Library.
 	# Arguments:
 	#  - sound_name => The filename of the desired sound.
-	proc loadSound {sound_name} {
+	proc loadSound {sound_name absolute_path} {
 		variable loaded_sounds
 
 		if { [info exists loaded_sounds($sound_name)] } {
 			return snd_$sound_name
 		}
 
-		snack::sound snd_$sound_name -file [::skin::GetSkinFile sounds $sound_name]
-		set loaded_sounds($sound_name) 1
+		if { $absolute_path == 1 } {
+			snack::sound snd_$sound_name -file $sound_name
+		} else {
+			snack::sound snd_$sound_name -file [::skin::GetSkinFile sounds $sound_name]			
+		}
+		set loaded_sounds($sound_name) $absolute_path
 
 		return snd_$sound_name
 	}
@@ -353,7 +469,7 @@ namespace eval ::skin {
 		}
 
 		# Now reload special images that need special treatment
-		variable loaded_images
+		global loaded_images
 		if {[info exists loaded_images(displaypicture_std_none)]} {
 			unset loaded_images(displaypicture_std_none)
 			::skin::getNoDisplayPicture $skin_name
@@ -366,12 +482,17 @@ namespace eval ::skin {
 		# Reload sounds
 		variable loaded_sounds
 		foreach name [array names loaded_sounds] {
-			snd_$name configure -file [::skin::GetSkinFile sounds $name]
+			if { [set loaded_sounds($name)] == 1 } {
+				snd_$name configure -file $name
+			} else {
+				snd_$name configure -file [::skin::GetSkinFile sounds $name]
+			}
 		}
-
-		# Change frame color (For All Platforms except Mac)
-		if {[catch {tk windowingsystem} wsystem] || $wsystem != "aqua"} {
-			catch {.main configure -background [::skin::getKey mainwindowbg]}
+		
+		# Change frame color
+		catch {.main configure -background [::skin::getKey mainwindowbg]}
+		
+		if { ![OnMac] } {
 			#We are not using pixmapscroll on Mac OS X
 			# Reload pixmapscroll's images
 			set psdir [LookForExtFolder $skin_name "pixmapscroll"]
@@ -381,6 +502,8 @@ namespace eval ::skin {
 			}	
 		}
 
+		::Event::fireEvent changedSkin gui
+
 
 	}
 
@@ -422,6 +545,9 @@ namespace eval ::skin {
 		sxml::register_routine $skin_id "skin:General:Colors" ::skin::SetConfigKeys
 		sxml::register_routine $skin_id "skin:General:Geometry" ::skin::SetConfigKeys
 		sxml::register_routine $skin_id "skin:General:Options" ::skin::SetConfigKeys
+		sxml::register_routine $skin_id "skin:Login:Colors" ::skin::SetConfigKeys
+		sxml::register_routine $skin_id "skin:Login:Geometry" ::skin::SetConfigKeys
+		sxml::register_routine $skin_id "skin:Login:Options" ::skin::SetConfigKeys
 		sxml::register_routine $skin_id "skin:ContactList:Colors" ::skin::SetConfigKeys
 		sxml::register_routine $skin_id "skin:ContactList:Geometry" ::skin::SetConfigKeys
 		sxml::register_routine $skin_id "skin:ContactList:Options" ::skin::SetConfigKeys
@@ -430,6 +556,7 @@ namespace eval ::skin {
 		sxml::register_routine $skin_id "skin:ChatWindow:Options" ::skin::SetConfigKeys
 		sxml::register_routine $skin_id "skin:smileys:emoticon" ::smiley::newEmoticon
 		sxml::register_routine $skin_id "skin:smileys:size" ::skin::SetEmoticonSize
+		sxml::register_routine $skin_id "skin:Fonts:Font" ::skin::newFont
 		sxml::parse $skin_id
 		sxml::end $skin_id
 		unset ::loading_skin
@@ -482,9 +609,8 @@ namespace eval ::skin {
 		}
 
 		set locations [list]
-		lappend locations 
 		#Get file using global path
-		if { "[string range $filename 0 0]" == "/" } {
+		if { "[string range $filename 0 0]" == "/" || "[string range $filename 1 2]" == ":/" } {
 			lappend locations ""
 		}
 		#Get from personal profile folder (needed for displaypics)
@@ -498,14 +624,13 @@ namespace eval ::skin {
 		#Get file from ~/.amsn/amsn-extras/skins folder
 		lappend locations [file join $HOME2 amsn-extras skins $skin $type]
 		#Get file from default skin
-                if { [::skin::getKey ignoredefaultsmileys] != 1 || $type != "smileys" } {
-                        lappend locations [file join [set ::program_dir] skins $defaultskin $type]
-                        #Get file from fallback location
-                        if { ($fblocation != "") } {
-                                lappend locations [file join $fblocation]
-                        }
-                }
-
+		if { [::skin::getKey ignoredefaultsmileys] != 1 || $type != "smileys" } {
+			lappend locations [file join [set ::program_dir] skins $defaultskin $type]
+			#Get file from fallback location
+			if { ($fblocation != "") } {
+				lappend locations [file join $fblocation]
+			}
+		}
 
 		foreach location $locations {
 			foreach extension $ext {
@@ -545,18 +670,6 @@ namespace eval ::skin {
 			return 0
 		}
 	}
-		
-	
-	
-
-
-
-
-
-
-
-
-
 
 
 	#We already have theses 2 procs in protocol.tcl but skins.tcl is loaded before protocol.tcl 
@@ -600,14 +713,17 @@ namespace eval ::skin {
 	proc FindSkins { } {
 		global HOME HOME2
 
-		set skins [glob -directory skins */settings.xml]
+		set skins [glob -nocomplain -directory skins */settings.xml]
 		set skins_in_home [glob -nocomplain -directory [file join $HOME skins] */settings.xml]
 		set skins_in_home2 [glob -nocomplain -directory [file join $HOME2 skins] */settings.xml]
 		set skins_in_extras [glob -nocomplain -directory [file join $HOME2 amsn-extras skins] */settings.xml]
-		set skins [concat $skins $skins_in_home $skins_in_home2 $skins_in_extras]
+		set skins [concat $skins $skins_in_home2 $skins_in_extras]
+		if { $HOME != $HOME2 } {
+			set skins [concat $skins $skins_in_home]
+		}
 		set skinlist [list]
 
-		foreach skin [lsort -dictionary $skins] {
+		foreach skin $skins {
 			set dir [file dirname $skin]
 			set desc ""
 
@@ -624,7 +740,7 @@ namespace eval ::skin {
 			lappend skinlist $skinname
 		}
 
-		return $skinlist
+		return [lsort -dictionary $skinlist ]
 	}
 
 
@@ -639,7 +755,7 @@ namespace eval ::skin {
 		}
 
 		# This bits are used to override certain keys loaded before with specific values for MacOS X (TkAqua)
-	#	if { ![catch {tk windowingsystem} wsystem] && $wsystem == "aqua" } {
+	#	if { [OnMac] } {
 	#		if { [info exists sdata(${cstack}:chatwindowbg)] } { ::skin::setKey buttonbarbg [string trim $sdata(${cstack}:chatwindowbg)] }
 	#			::skin::setKey chat_top_border 0
 	#			::skin::setKey chat_output_border 0
@@ -666,6 +782,181 @@ namespace eval ::skin {
 		# Procedures binded to the XML parser must ALWAYS return 0
 		return 0
 	}
+	
+	################################################################
+	# ::skin::SetPixmapNames
+	# Setup the pixmap proxy using default filenames. Moved here from cmsn_draw_main.
+	proc SetPixmapNames { } {
+		::skin::setPixmap amsnicon amsnicon.png
+
+		::skin::setPixmap msndroid msnbot.gif
+		::skin::setPixmap online online.gif
+		::skin::setPixmap offline offline.gif
+		::skin::setPixmap away away.gif
+		::skin::setPixmap busy busy.gif
+		::skin::setPixmap mobile mobile.gif
+
+		::skin::setPixmap info info.gif
+
+		::skin::setPixmap space_update space_update.gif
+		::skin::setPixmap space_noupdate space_noupdate.gif
+		::skin::setPixmap spaces_dock spaces_dock.gif
+		::skin::setPixmap spaces_undock spaces_undock.gif
+	
+		::skin::setPixmap bonline bonline.gif
+		::skin::setPixmap boffline boffline.gif
+		::skin::setPixmap baway baway.gif
+		::skin::setPixmap bbusy bbusy.gif
+		::skin::setPixmap mystatus_bg mystatus_bg.gif
+		::skin::setPixmap mystatus_bg_hover mystatus_bg_hover.gif
+		::skin::setPixmap nonim nonim.gif
+	
+		::skin::setPixmap mailbox mailbox.gif
+		::skin::setPixmap mailbox_new mailbox_unread.gif
+	
+		::skin::setPixmap contract contract.gif
+		::skin::setPixmap contract_hover contract_hover.gif
+		::skin::setPixmap expand expand.gif
+		::skin::setPixmap expand_hover expand_hover.gif
+		::skin::setPixmap grouptopback_dark grouptopback_dark.gif
+		::skin::setPixmap grouptopback_light grouptopback_light.gif
+		
+		::skin::setPixmap globe globe.gif
+		::skin::setPixmap download download.gif
+		::skin::setPixmap warning warning.gif
+	
+		::skin::setPixmap button button.gif
+		::skin::setPixmap button_hover button_hover.gif
+		::skin::setPixmap button_pressed button_pressed.gif
+		::skin::setPixmap button_disabled button_disabled.gif
+		::skin::setPixmap button_focus button_focus.gif
+
+		::skin::setPixmap checkbox checkbox.gif
+		::skin::setPixmap checkbox_on checkbox_on.gif
+	
+		::skin::setPixmap typingimg typing.gif
+		::skin::setPixmap miniinfo miniinfo.gif
+		::skin::setPixmap miniwarning miniwarn.gif
+		::skin::setPixmap minijoins minijoins.gif
+		::skin::setPixmap minileaves minileaves.gif
+	
+		::skin::setPixmap cwtopback cwtopback.gif
+		::skin::setPixmap camicon camicon.gif	
+		
+	
+		::skin::setPixmap butsmile butsmile.gif
+		::skin::setPixmap butsmile_hover butsmile_hover.gif
+		::skin::setPixmap butfont butfont.gif
+		::skin::setPixmap butfont_hover butfont_hover.gif
+		::skin::setPixmap butvoice butvoice.gif
+		::skin::setPixmap butvoice_hover butvoice_hover.gif
+		::skin::setPixmap butblock butblock.gif
+		::skin::setPixmap butblock_hover butblock_hover.gif
+		::skin::setPixmap butsend butsend.gif
+		::skin::setPixmap butsend_hover butsend_hover.gif
+		::skin::setPixmap butinvite butinvite.gif
+		::skin::setPixmap butinvite_hover butinvite_hover.gif
+		::skin::setPixmap butwebcam butwebcam.gif
+		::skin::setPixmap butwebcam_hover butwebcam_hover.gif
+		::skin::setPixmap butcall butcall.gif
+		::skin::setPixmap butcall_hover butcall_hover.gif
+		::skin::setPixmap butcallvideo butcallvideo.gif
+		::skin::setPixmap butcallvideo_hover butcallvideo_hover.gif
+		::skin::setPixmap buthangup buthangup.gif
+		::skin::setPixmap buthangup_hover buthangup_hover.gif
+		::skin::setPixmap buthangupvideo buthangupvideo.gif
+		::skin::setPixmap buthangupvideo_hover buthangupvideo_hover.gif
+		::skin::setPixmap buthangup_mini buthangup_mini.gif
+		::skin::setPixmap butnewline newline.gif
+		::skin::setPixmap loginbutton loginbut.gif
+		::skin::setPixmap loginbutton_hover loginbut_hover.gif
+		::skin::setPixmap sendbutton sendbut.gif
+		::skin::setPixmap sendbutton_hover sendbut_hover.gif
+		::skin::setPixmap imgshow imgshow.gif
+		::skin::setPixmap imgshow_hover imgshow_hover.gif
+		::skin::setPixmap imghide imghide.gif
+		::skin::setPixmap imghide_hover imghide_hover.gif
+		::skin::setPixmap confbut confbut.gif
+		::skin::setPixmap confbuth confbuth.gif
+		::skin::setPixmap stopbut stopbut.gif
+		::skin::setPixmap stopbuth stopbuth.gif
+		::skin::setPixmap pausebut pausebut.gif
+		::skin::setPixmap pausebuth pausebuth.gif
+		::skin::setPixmap playbut playbut.gif
+		::skin::setPixmap playbuth playbuth.gif
+		::skin::setPixmap recordbut recordbut.gif
+		::skin::setPixmap recordbuth recordbuth.gif
+		::skin::setPixmap pause pause.gif
+	
+		::skin::setPixmap ring ring.gif
+		::skin::setPixmap ring_disabled ring_disabled.gif
+		
+		::skin::setPixmap winwritecam cam_in_chatwin.png
+	
+		::skin::setPixmap webcam webcam.png
+		::skin::setPixmap camempty camempty.png
+		::skin::setPixmap yes-emblem yes-emblem.gif
+		::skin::setPixmap no-emblem no-emblem.gif
+	
+		#@@@@@@@@ webMSN DP
+		::skin::setPixmap webmsn_dp webmsn_dp.gif
+		::skin::setPixmap webmsn webmsn.gif
+	
+		::skin::setPixmap sipicon sipicon.gif
+		::skin::setPixmap fticon fticon.gif
+		::skin::setPixmap ftreject ftreject.gif
+	
+		::skin::setPixmap notifico notifico.gif
+		::skin::setPixmap notifclose notifclose.gif
+		::skin::setPixmap notifyonline notifyonline.gif
+		::skin::setPixmap notifyoffline notifyoffline.gif
+		::skin::setPixmap notifyplugins notifyplugins.gif
+		::skin::setPixmap notifystate notifystate.gif
+		::skin::setPixmap notifymsg notifymsg.gif
+		::skin::setPixmap notifyemail notifyemail.gif
+	
+		::skin::setPixmap blocked blocked.gif
+		::skin::setPixmap blocked_off blocked_off.gif
+		::skin::setPixmap colorbar colorbar.gif
+	
+		::skin::setPixmap bell bell.gif
+		::skin::setPixmap belloff belloff.gif
+	
+		::skin::setPixmap notinlist notinlist.gif
+		::skin::setPixmap note note.gif
+		::skin::setPixmap smile smile.gif
+	
+		::skin::setPixmap loganim loganim.gif
+
+		::skin::setPixmap smallanim smallanim.gif
+	
+		::skin::setPixmap greyline greyline.gif
+	
+		::skin::setPixmap assistant assistant.gif
+		::skin::setPixmap assistant_audio assistant_audio.gif
+		::skin::setPixmap assistant_webcam assistant_webcam.gif
+		
+		::skin::setPixmap voice_icon voice.gif
+
+		::skin::setPixmap ampli ampli.gif
+		::skin::setPixmap ampli_pressed ampli_pressed.gif
+		::skin::setPixmap mic mic.gif
+		::skin::setPixmap mic_muted mic_muted.gif
+		::skin::setPixmap speaker speaker.gif
+		::skin::setPixmap speaker_muted speaker_muted.gif
+
+		::skin::setPixmap nullimage null
+		#set the nullimage transparent
+		[::skin::loadPixmap nullimage] blank
+		
+		if { [OnMac] } {
+			::skin::setPixmap logolinmsn logomacmsn.gif
+			::skin::setPixmap arrow arrowmac.gif
+		} else {
+			::skin::setPixmap logolinmsn logolinmsn.gif
+			::skin::setPixmap arrow arrow.gif
+		}
+	}
 }
 
 namespace eval ::skinsGUI {
@@ -683,66 +974,70 @@ namespace eval ::skinsGUI {
 			return
 		}
 
+#TODO: the preview pane should be fixed size and scrollable so the window doesn't get overpopulated or too big
+
 		toplevel $w
-		if {[OnMac]} {
+#		if {[OnMac]} {
 			wm resizable $w 1 1
-		} else {
-			wm resizable $w 0 0
-		}
+#		} else {
+#			wm resizable $w 0 0
+#		}
 		wm title $w "[trans chooseskin]"
 		wm geometry $w +100+100
 
-		label $w.choose -text "[trans chooseskin]" -font bboldf
-		pack $w.choose -side top
+		frame $w.top
+		label $w.top.choose -text "[trans chooseskin]" -font bboldf
 
-		label $w.restart -text "[trans restartforskin]" -fg #FF0000
-                pack $w.restart -side top
+		label $w.top.restart -text "[trans restartforskin]"
 
-		frame $w.main -relief solid -borderwidth 2
-		frame $w.main.left -relief flat
-		frame $w.main.right -relief flat
-		frame $w.main.left.images -relief flat
-		text $w.main.left.desc -height 6 -width 40 -relief flat -background [::skin::getKey mainwindowbg] \
+		frame $w.bottom
+		frame $w.main
+		frame $w.main.left
+		frame $w.main.right
+		frame $w.main.left.images
+		text $w.main.left.desc -height 6 -width 40 -relief sunken \
 			-font sboldf -wrap word
-		listbox $w.main.right.box -yscrollcommand "$w.main.right.ys set" -font splainf -background \
-			white -relief flat -highlightthickness 0  -height 8 -width 30
+		listbox $w.main.right.box -yscrollcommand "$w.main.right.ys set" -height 8 -width 30
 		scrollbar $w.main.right.ys -command "$w.main.right.box yview" -highlightthickness 0 \
 			-borderwidth 1 -elementborderwidth 2
 
 		bind $w <<Escape>> "::skinsGUI::SelectSkinCancel $w"
 
+
+		label $w.bottom.status -text ""
+
+		set select -1
+		set idx 0
+
+		label $w.bottom.getmore -text "[trans getmoreskins]"  -font splainf \
+			-bg [::skin::getKey extrastdwindowcolor] -fg [::skin::getKey extralinkcolor]
+		bind $w.bottom.getmore <Enter> "$w.bottom.getmore configure -font sunderf -cursor hand2 \
+			-bg [::skin::getKey extralinkbgcoloractive] -fg [::skin::getKey extralinkcoloractive]"
+		bind $w.bottom.getmore <Leave> "$w.bottom.getmore configure -font splainf -cursor left_ptr \
+			-background [::skin::getKey extrastdwindowcolor] -foreground [::skin::getKey extralinkcolor]"
+		bind $w.bottom.getmore <ButtonRelease> "launch_browser $::weburl/skins.php"
+
+		button $w.bottom.ok -text "[trans ok]" -command "::skinsGUI::SelectSkinOk $w"
+		button $w.bottom.cancel -text "[trans cancel]" -command "::skinsGUI::SelectSkinCancel $w"
+		checkbutton $w.bottom.preview -text "[trans preview]" -variable ::skin::preview_skin_change -onvalue 1 -offvalue 0
+
+                pack $w.top -side top -fill x
+		pack $w.bottom -side bottom -fill x
+		pack $w.main -side top -fill both -expand 1
+
+		pack $w.bottom.status -side bottom
+		pack $w.bottom.getmore -side left -padx 5
+		pack $w.bottom.ok  $w.bottom.cancel $w.bottom.preview -side right -pady 5 -padx 5
 		pack $w.main.left.images -in $w.main.left -side top -expand 0 -fill both
 		pack $w.main.left.desc -in $w.main.left -side bottom -expand 1 -fill both
 		pack $w.main.left -in $w.main -side left -expand 1 -fill both
 		pack $w.main.right.ys -side right -fill both
 		pack $w.main.right.box -side left -expand 0 -fill both
 		pack $w.main.right -side right -expand 1 -fill both
-		pack $w.main -expand 1 -fill both
+                pack $w.top.restart -side top
+		pack $w.top.choose -side top
 
-		label $w.status -text ""
-		pack $w.status -side bottom
 
-#		image create photo blank -width 1 -height 75
-#		label $w.main.left.images.blank -image blank
-
-#		image create photo blank2 -width 400 -height 1
-#		label $w.main.left.images.blank2 -image blank2
-
-		set select -1
-		set idx 0
-
-		label $w.getmore -text "[trans getmoreskins]" -fg #0000FF
-
-		button $w.ok -text "[trans ok]" -command "::skinsGUI::SelectSkinOk $w" 
-		button $w.cancel -text "[trans cancel]" -command "::skinsGUI::SelectSkinCancel $w" 
-		checkbutton $w.preview -text "[trans preview]" -variable ::skin::preview_skin_change -onvalue 1 -offvalue 0
-		
-		pack $w.getmore -side left -padx 5
-	        bind $w.getmore <Enter> "$w.getmore configure -font sunderf"
-	        bind $w.getmore <Leave> "$w.getmore configure -font splainf"
-		bind $w.getmore <ButtonRelease> "launch_browser $::weburl/skins.php"			
-		
-		pack $w.ok  $w.cancel $w.preview -side right -pady 5 -padx 5
 
 		set the_skins [::skin::FindSkins]
 
@@ -758,7 +1053,9 @@ namespace eval ::skinsGUI {
 			status_log "selecy = 0 --- didn't find current skin defaulting to first"
 
 			$w.main.right.box selection set $select
-	 		$w.main.right.box itemconfigure $select -background #AAAAAA
+	 		$w.main.right.box itemconfigure $select \
+				-bg [::skin::getKey extralistboxselectedbg] -fg [::skin::getKey extralistboxselected] \
+				-selectforeground [::skin::getKey extralistboxselected]
 
 			set currentskin [lindex [lindex $the_skins 0] 0]
 			if { $::skin::preview_skin_change == 1 } {
@@ -769,12 +1066,14 @@ namespace eval ::skinsGUI {
 			status_log "select = $select --- [::config::getGlobalKey skin]\n"
 
 			$w.main.right.box selection set $select
-			$w.main.right.box itemconfigure $select -background #AAAAAA
+			$w.main.right.box itemconfigure $select \
+				-bg [::skin::getKey extralistboxselectedbg] -fg [::skin::getKey extralistboxselected] \
+				-selectforeground [::skin::getKey extralistboxselected]
 		}
 	
 		::skinsGUI::DoPreview 1
 		bind $w <Destroy> "grab release $w"
-		bind $w.main.right.box <Button1-ButtonRelease> "::skinsGUI::DoPreview"
+		bind $w.main.right.box <<Button1>> "::skinsGUI::DoPreview"
 
 		moveinscreen $w 30
 	}
@@ -872,9 +1171,9 @@ namespace eval ::skinsGUI {
 	#  - w => Path of the widget skin selector.
 	proc SelectSkinOk { w } {
 		if { [$w.main.right.box curselection] == "" } {
-			$w.status configure -text "[trans selectskin]"
+			$w.bottom.status configure -text "[trans selectskin]"
 		} else {
-			$w.status configure -text ""
+			$w.bottom.status configure -text ""
 			set skinidx [$w.main.right.box curselection]
 			set skin [lindex [lindex [::skin::FindSkins] $skinidx] 0]
 			status_log "Chose skin No $skinidx : $skin\n"
