@@ -1,6 +1,5 @@
-#
-# $Id: smileys.tcl 7047 2006-07-13 21:58:17Z thaven $
-#
+
+::Version::setSubversionId {$Id: smileys.tcl 11709 2009-10-15 21:14:49Z kakaroto $}
 
 if { $initialize_amsn == 1 } {
 	global smileys_drawn
@@ -48,12 +47,12 @@ namespace eval ::smiley {
 		if { ! [info exists sdata(${cstack}:file)] } { return 0 }
 		if { [info exists sdata(${cstack}:disabled)] && [is_true $sdata(${cstack}:disabled)] } { return 0 }
 
-                if { [::skin::getKey ignoredefaultsmileys] == 1 && ! [info exists emotions_cleared] } {
-                        set emotions_cleared 1
-                        unset emotions_data
-                        unset emotions_names
-                        unset emotions
-                }
+		if { [::skin::getKey ignoredefaultsmileys] == 1 && ! [info exists emotions_cleared] } {
+			set emotions_cleared 1
+			unset emotions_data
+			unset emotions_names
+			unset emotions
+		}
 		
 		#Get the smiley info: name, text and file, and other
 		#existing fields, like sound or animated. Store them
@@ -143,6 +142,32 @@ namespace eval ::smiley {
 	}
 	
 		
+	# This function will resize those huge custom emoticons to the standard maximum size of 50x50 accepted by WLM.
+	proc resizeCustomSmiley { image } {
+		if {[catch {image width $image}] } {
+			status_log "asked to resize custom emoticon $image that doesn't exist" red
+			return
+		}
+		if { [image width $image] > 50 || [image height $image] > 50 } {
+			set width [image width $image]
+			set height [image height $image]
+			if {$width > 50} {
+				set height [expr {int(((1.0*$height)/$width) * 50)}]
+				set width 50
+				if {$height < 19} {
+					set height 19
+				}
+			} else {
+				set width [expr {int(((1.0*$width)/$height) * 50)}]
+				set height 50
+				if {$width < 19} {
+					set width 19
+				}
+			}
+			::picture::Resize $image $width $height
+		}
+	}
+
 	#///////////////////////////////////////////////////////////////////////////////
 	# proc newCustomEmoticonXML {cstack cdata saved_data cattr saved_attr args}
 	#
@@ -171,24 +196,35 @@ namespace eval ::smiley {
 			set emotion($field_sort) [string trim $sdata($field)]
 		}
 
-
 		#Create the image now and store it   ;# $emotion(name) is unique as we store the emoticon in the array under this name (cfr infra)
-		if { [catch { image create photo emoticonCustom_std_$emotion(text) -file $emotion(file) -format cximage } emotion(image_name) ] } {
+		if { [catch { image create photo emoticonCustom_std_$emotion(text) -file [PathRelToAbs $emotion(file)] -format cximage } emotion(image_name) ] } {
 			status_log "Error when creating image for emoticon $emotion(name) : $emotion(image_name)"
-			#Error when creating smiley's image so we don't add it
-			return 0
+
+			#error loading emoticon, just flag it as unreachable
+			set emotion(reachable) 0
+		} else {
+			set emotion(reachable) 1
+
+			# Make sure the smiley is max 50x50
+			::smiley::resizeCustomSmiley emoticonCustom_std_$emotion(text)
+			
+			update
 		}
 		
 		#Store the emoticon data in the custom_emoticons array
 		set custom_emotions($emotion(name)) [array get emotion]
-
-		variable sortedemotions
-		if {[info exists sortedemotions]} { unset sortedemotions }
-		
 		
 		return 0
 	}
 	
+	#///////////////////////////////////////////////////////////////////////////////
+	# proc cleanup
+	#
+	# Cleans up the smiley storage
+	proc cleanup {} {
+		variable sortedemotions
+		if {[info exists sortedemotions]} { unset sortedemotions }
+	}
 
 	
 	#///////////////////////////////////////////////////////////////////////////////
@@ -212,7 +248,7 @@ namespace eval ::smiley {
 			
 			#Add our custom smileys. For custom smileys we also add
 			#the smiley name associated to that symbol
-			foreach name [array names custom_emotions] {
+		    	foreach name [array names custom_emotions] {
 			
 				if { ![info exists custom_emotions($name)] } {
 					status_log "substYourSmileys: Custom smiley $name doesn't exist in custom_emotions array!!\n" red
@@ -271,6 +307,11 @@ namespace eval ::smiley {
 					#Get all emoticon data from custom_emotions array
 					array set emotion $custom_emotions($name)
 					
+					#check if emoticon is unreachable and skip it
+					if { $emotion(reachable) == 0 } {
+						continue
+					}
+					
 					if { [info exists emotion(casesensitive)] && [is_true $emotion(casesensitive)]} {
 						set nocase "-exact"
 					} else {
@@ -279,10 +320,10 @@ namespace eval ::smiley {
 					
 					set animated [expr {[info exists emotion(animated)] && [is_true $emotion(animated)]}]
 					if { $contact_list == 0 && [info exists emotion(sound)] && $emotion(sound) != "" } {
-						set sound $emotion(sound)
+						set sound [PathRelToAbs $emotion(sound)]
 					} else { set sound "" }
 					set image_name $emotion(image_name)
-					set image_file $emotion(file)
+					set image_file [PathRelToAbs $emotion(file)]
 					
 					
 					array unset emotion
@@ -316,26 +357,26 @@ namespace eval ::smiley {
 				set start [$tw index ${pos}+1c]
 				set endpos [$tw index $pos+[string length $symbol]c]
 
-				#set skip 0
+				set skip 0
 				# used to avoid invalid search caused by elided text, such as in ::op
 				if { [string equal -nocase [$tw get $pos $endpos]  $symbol] == 0} {
 					continue
-				#} else {
-				#	for { set i $pos } {$i <= $endpos && !$skip} {set i [$tw index $i+1c]} {
-				#		set tags [$tw tag names $i]
-				#		foreach tag_name $tags {
-				#			set elided [$tw tag cget $tag_name -elide]
-				#			if {$elided != "" && $elided} {
-				#				set skip 1
-				#				break;
-				#			}
-				#		}
-				#	}
+				} else {
+					for { set i $pos } {$i < $endpos && !$skip} {set i [$tw index $i+1c]} {
+						set tags [$tw tag names $i]
+						foreach tag_name $tags {
+							set elided [$tw tag cget $tag_name -elide]
+							if {$elided != "" && $elided} {
+								set skip 1
+								break;
+							}
+						}
+					}
 				}
 				
-				#if { $skip } {
-				#	continue
-				#}
+				if { $skip } {
+					continue
+				}
 
 				set start [::smiley::SubstSmiley $tw $pos $symbol $image_name $image_file $animated $sound]
 				#If SubstSmiley returns -1, start from beggining.
@@ -398,17 +439,16 @@ namespace eval ::smiley {
 		
 		set w .smile_selector
 		
-		
 		if { ! [winfo exists $w]} { CreateSmileyMenu }
 		
 		wm state $w normal
 		set x [expr {$x - 15}]
 		set y [expr {$y + 15 - [winfo reqheight $w]}]
 		wm geometry $w +$x+$y
+
 		#It won't work on Windows without this
 		update idletasks
-		
-		
+				
 		#It won't work on Windows without this
 		raise $w
 		
@@ -418,18 +458,14 @@ namespace eval ::smiley {
 			if { [ValueForSmiley $name hiden 1] } {continue}
 		
 			#Get the first symbol for that smiley
-			set symbol [lindex [ValueForSmiley $name symbols] 0]
+			set symbol [string map { % %% } [lindex [ValueForSmiley $name symbols] 0]]
 			
 			#This must be cached due to a race condition (if you double click
 			#the smileys menu for first time, the second click can launch
 			#this procedure without all smileys having been created
 			catch { 
 				#TODO: Improve this now we know about quoting a bit more?
-				if { [string match {(%)} $symbol] != 0 } {
-					bind $w.c.$temp <Button1-ButtonRelease> "catch {$text insert insert \{(%%)\}; wm state $w withdrawn} res"
-				} else {
-					bind $w.c.$temp <Button1-ButtonRelease> "catch {[list $text insert insert $symbol]\; wm state $w withdrawn} res" 
-				}
+				bind $w.c.$temp <<Button1>> "catch {[list $text insert insert $symbol]\; wm state $w withdrawn; event generate $text <KeyRelease>} res" 
 			}
 			
 			incr temp
@@ -437,33 +473,38 @@ namespace eval ::smiley {
 		
 		#Now add custom emotions bindings
 		global custom_emotions
-		foreach name [array names custom_emotions] {
+	    	foreach name [lsort [array names custom_emotions]] {
 			array set emotion $custom_emotions($name)
-			set symbol [lindex $emotion(text) 0]
+			set symbol [string map { % %% } [lindex $emotion(text) 0]]
 			
 			#This must be cached due to a race condition (if you double click
 			#the smileys menu for first time, the second click can launch
 			#this procedure without all smileys having been created
 			catch { 
-				#TODO: Improve this now we know about quoting a bit more?
-				if { [string match {(%)} $symbol] != 0 } {
-					bind $w.c.$temp <Button1-ButtonRelease> "catch {$text insert insert \{(%%)\}; wm state $w withdrawn} res"
-				} else {
-					bind $w.c.$temp <Button1-ButtonRelease> "catch {[list $text insert insert $symbol]\; wm state $w withdrawn} res" 
+				if { $emotion(reachable) == 1 } {
+					#TODO: Improve this now we know about quoting a bit more?
+					bind $w.c.$temp <<Button1>> "catch {[list $text insert insert $symbol]\; wm state $w withdrawn; event generate $text <KeyRelease>} res" 
 				}
 				#Add binding for custom emoticons
-				if {![catch {tk windowingsystem} wsystem] && $wsystem == "aqua"} {
-					
-					bind $w.c.$temp <Button2-ButtonRelease> [list ::smiley::editCustomEmotion $name]
-					bind $w.c.$temp <Control-ButtonRelease> [list ::smiley::editCustomEmotion $name]
-				} else {
-					bind $w.c.$temp <Button3-ButtonRelease> [list ::smiley::editCustomEmotion $name]
-				}
+				bind $w.c.$temp <<Button3>> [list ::smiley::editCustomEmotion $name]
 			}
 			
 			incr temp
 		}
+
+	   	# Here, I disable the "Leave" event so that in case the window 
+	    	# is in the upper left corner and really small, when we click, 
+	    	# and the mouse is already inside the window, we don't want the
+	    	# 'moveinscreen' to generate a <Leave> event and thus making it
+	    	# impossible to show the smiley window
+
+	    	set binding [bind $w <Leave>]
+	    	bind $w <Leave> ""
+
+		moveinscreen $w 5
 		
+		bind $w <Leave> $binding
+
 		event generate $w <Enter>
 	
 	}
@@ -471,20 +512,28 @@ namespace eval ::smiley {
 
 	#//////////////////////////////////////////////////////////////////
 	#proc that changes a string into a list with seperated text/smileys
-	proc parseMessageToList { name {contact_list 0} {include_custom 0} } {
+	# TODO : WTF! This is a copy paste of substSmiley.. code should NOT be duplicated!
+	# TODO : contact_list variable is about playing sound, not about including smileys..
+	proc parseMessageToList { name {contact_list 0} {play_sounds 0} {include_custom 0} } {
 		global emotions
 		global custom_emotions
 		variable sortedemotions
 
-		#This is a poor place for sort smileys should be done only when adding new smileys to the list?
-		#SortSmileys
+		SortSmileys
 
-		set l [list [ list "text" "$name" ]]
-		set llength 1
+		set llength [llength $name]
+		set l $name
 
 		#Search for all possible emotions, after they are sorted by symbol length
-		SortSmileys
-		foreach emotion_data [concat $sortedemotions [list [list "\n" "newline"]]] {
+		set foreachlist [list ]
+
+		if { [::config::getKey listsmileys] || !$contact_list } {
+			set foreachlist [concat $foreachlist $sortedemotions]
+		}
+
+		set foreachlist [lappend foreachlist [list "\n" "newline"]]
+
+		foreach emotion_data $foreachlist {
 
 			#Symbol is first element
 			set symbol [lindex $emotion_data 0]
@@ -502,20 +551,25 @@ namespace eval ::smiley {
 					#Get all emoticon data from custom_emotions array
 					array set emotion $custom_emotions($name)
 
+					#check if emoticon is unreachable and skip it
+					if { $emotion(reachable) == 0 } {
+						continue
+					}
+					
 					if { [info exists emotion(casesensitive)] && [is_true $emotion(casesensitive)]} {
-						set nocase "-exact"
+						set nocase 0
 					} else {
-						set nocase "-nocase"
+						set nocase 1
 					}
 		
 					set animated [expr {[info exists emotion(animated)] && [is_true $emotion(animated)]}]
-					if { $contact_list == 0 && [info exists emotion(sound)] && $emotion(sound) != "" } {
-						set sound $emotion(sound)
+					if { $play_sounds && [info exists emotion(sound)] && $emotion(sound) != "" } {
+						set sound [PathRelToAbs $emotion(sound)]
 					} else {
 						set sound ""
 					}
 					set image_name $emotion(image_name)
-					set image_file $emotion(file)
+					set image_file [PathRelToAbs $emotion(file)]
 		
 		
 					array unset emotion
@@ -523,7 +577,7 @@ namespace eval ::smiley {
 		
 			} elseif { $smiley_type == "newline" } {
 				set emotion_name "newline"
-				set nocase "-exact"
+				set nocase 0
 				set animated 0
 				set sound ""
 				set image_name "__newline__"
@@ -533,13 +587,13 @@ namespace eval ::smiley {
 				set emotion_name $emotions($symbol)
 	
 				if { [ValueForSmiley $emotion_name casesensitive 1] } {
-					set nocase "-exact"
+					set nocase 0
 				} else {
-					set nocase "-nocase"
+					set nocase 1
 				}
 	
 				set animated [ValueForSmiley $emotion_name animated 1]
-				if { $contact_list == 0 && [ValueForSmiley $emotion_name sound] != "" } {
+				if { $play_sounds && [ValueForSmiley $emotion_name sound] != "" } {
 					set sound [ValueForSmiley $emotion_name sound]
 				} else {
 					set sound ""
@@ -550,21 +604,23 @@ namespace eval ::smiley {
 
 			set listpos 0
 			#Keep searching until no matches
-#TODO: make it not case sensitive
+
 			while { $listpos < $llength } {
 				if { ([lindex $l $listpos 0] != "text") } {
 					incr listpos
 					continue
 				}
 				
-				if {[set pos [string first $symbol [lindex $l $listpos 1]]] != -1 } {
+				if {[set pos [expr $nocase ? \
+					[string first [string tolower $symbol] [string tolower [lindex $l $listpos 1]]] : \
+					[string first $symbol [lindex $l $listpos 1]]]] != -1 } {
 					set p1 [string range [lindex $l $listpos 1] 0 [expr {$pos - 1}]]
 					set p2 $image_name
 					set p3 [string range [lindex $l $listpos 1] [expr {$pos + [string length $symbol]}] end]
 
 #TODO: #need to change for an 'in-place' lreplace (here and below)
 					if { $p2 == "__newline__" } {
-						set l [lreplace $l $listpos $listpos [list text $p1] [list "newline"] [list text $p3] ]
+						set l [lreplace $l $listpos $listpos [list text $p1] [list "newline" "\n"] [list text $p3] ]
 					} else {
 						set l [lreplace $l $listpos $listpos [list text $p1] [list smiley $p2 $symbol] [list text $p3] ]
 					}
@@ -582,30 +638,31 @@ namespace eval ::smiley {
 						incr listpos -1
 					}
 				}
-				#text can never be followed by another set of text
-				incr listpos 2
+				incr listpos 1
 			}
 		}
 
-#TODO: also parse newlines as [list newline]
-
 		return $l
 	}
 
 
 	
 	#Create ONE smiley in the smileys menu
-	proc CreateSmileyInMenu {w cols rows smiw smih emot_num name symbol image file animated} {
+	proc CreateSmileyInMenu {w cols rows smiw smih emot_num name symbol image file animated {reachable 1}} {
 		catch {
-			#number of menu-item in name as I'm usure about the name being unique and the quoting of the symbol
-			set resized [image create photo emoticonAll_tny_${emot_num}_small]
-			$resized copy $image
-			if {[image width $image] > 22 && [image height $image] > 22} {
-				::picture::ResizeWithRatio $resized 22 22
+			if { $reachable == 1 } {
+				#number of menu-item in name as I'm usure about the name being unique and the quoting of the symbol
+				set resized [image create photo emoticonAll_tny_${emot_num}_small]
+				$resized copy $image
+				if {[image width $image] > 22 && [image height $image] > 22} {
+					::picture::ResizeWithRatio $resized 22 22
+				}
+				label $w.$emot_num -image $resized -background [$w cget -background]
+				bind $w.$emot_num <Destroy> "image delete $resized"
+			} else {
+				label $w.$emot_num -background red
 			}
-			label $w.$emot_num -image $resized -background [$w cget -background]
-			bind $w.$emot_num <Destroy> "image delete $resized"
-	
+				
 			$w.$emot_num configure -cursor hand2 -borderwidth 1 -relief flat
 			
 			#Bindings for raise/flat on mouse over
@@ -613,7 +670,13 @@ namespace eval ::smiley {
 			bind $w.$emot_num <Leave> [list $w.$emot_num configure -relief flat]
 
 			#Tooltip
-			if { [::config::getKey tooltips] } {set_balloon $w.$emot_num "$name\n[trans triggers]: $symbol" "$image"}
+			if { [::config::getKey tooltips] } {
+				if { $reachable == 1 } {
+					set_balloon $w.$emot_num "$name\n[trans triggers]: $symbol" "$image"
+				} else {
+					set_balloon $w.$emot_num "$name\n([trans smileynotavailable])\n[trans triggers]: $symbol"
+				}
+			}
 			set xpos [expr {($emot_num % $cols)* $smiw}]
 			set ypos [expr {($emot_num / $cols) * $smih}]
 			$w create window $xpos $ypos -window $w.$emot_num -anchor nw -width $smiw -height $smih
@@ -668,7 +731,7 @@ namespace eval ::smiley {
 		
 		
 		canvas $w.c -background white -borderwidth 0 -relief flat \
-			-selectbackground white -selectborderwidth 0 
+			-selectbackground white -selectborderwidth 0 -width $x_geo -height $y_geo
 		pack $w.c -expand true -fill both
 		
 		#Add standard smileys
@@ -690,14 +753,14 @@ namespace eval ::smiley {
 		
 		#Now add custom emotions
 		global custom_emotions
-		foreach name [array names custom_emotions] {
+		foreach name [lsort [array names custom_emotions]] {
 		
 			array set emotion $custom_emotions($name)
 			if {![info exists emotion(animated)]} { set emotion(animated) 0 }
 			set animated [expr {$emotion(animated) && [::config::getKey animatedsmileys 0]}]
 			
 			CreateSmileyInMenu $w.c $cols $rows $smiw $smih \
-				$emot_num $name [lindex $emotion(text) 0] $emotion(image_name) $emotion(file) $animated
+				$emot_num $name [lindex $emotion(text) 0] $emotion(image_name) [PathRelToAbs $emotion(file)] $animated $emotion(reachable)
 	
 			incr emot_num
 		}
@@ -706,13 +769,37 @@ namespace eval ::smiley {
 		label $w.c.custom_new -text "[trans custom_new]"  -background [$w.c cget -background] -font sboldf
 		bind $w.c.custom_new <Enter> [list $w.c.custom_new configure -relief raised]
 		bind $w.c.custom_new <Leave> [list $w.c.custom_new configure -relief flat]
-		bind $w.c.custom_new <Button1-ButtonRelease> "::smiley::newCustomEmoticonGUI; event generate $w <Leave>"
+		bind $w.c.custom_new <<Button1>> "::smiley::newCustomEmoticonGUI; wm state $w withdrawn"
 		
 		set ypos [expr {(($rows-1)*$smih + ($smih/2))}]
 		$w.c create window  0 $ypos -window $w.c.custom_new -width [expr {$x_geo - 2}] -height $smih -anchor w
 		
 		
-		bind $w <Enter> "bind $w <Leave> \"bind $w <Leave> \\\"wm state $w withdrawn\\\"\""
+		bind $w <Leave> "::smiley::handleLeaveEvent $w $x_geo $y_geo"
+	}
+
+
+	#///////////////////////////////////////////////////////////////////////////////
+	# proc handleLeaveEvent { w x_geo y_geo }
+	#
+	# Handle the Leave event, detecting if is truly generated from the mouse exiting
+	# from the menu, to work around the Compiz/Beryl bug
+
+	proc handleLeaveEvent { w x_geo y_geo } {
+		# Get the pointer and the window coordinates
+		set pointer_x [winfo pointerx $w]
+		set pointer_y [winfo pointery $w]
+		set window_x [winfo rootx $w]
+		set window_y [winfo rooty $w]
+		# Calculate pointer coordinates relative to the menu
+		set x [expr {$pointer_x-$window_x}]
+		set y [expr {$pointer_y-$window_y}]
+		# Check if the pointer is outside the menu
+		# The first two conditions refers to the case in which the pointe is no
+		# more on the same screen of the menu (and is therefore outside of it)
+		if { $pointer_x == -1 || $pointer_y==-1 || $x < 0 || $x > $x_geo || $y < 0 || $y > $y_geo } {
+			wm state $w withdrawn
+		}
 	}
 	
 	
@@ -821,8 +908,10 @@ namespace eval ::smiley {
 
 	proc NewCustomEmoticonGUI_Delete { name } {
 		global custom_emotions
-		
-		catch {image delete $name}
+	
+		array set emotion $custom_emotions($name)
+		catch {image delete $emotion(image_name)}
+		catch {image delete $emotion(preview)}
 
 		unset custom_emotions($name)
 		if { [winfo exists .smile_selector]} {destroy .smile_selector}
@@ -863,7 +952,12 @@ namespace eval ::smiley {
 		
 		foreach element [list name file animated sound casesensitive] {
 			if {[info exists emotion($element)]} {
-				set new_custom_cfg($element) $emotion($element)
+				if {$element == "file" || $element == "sound"} {
+					#restore absolute path
+					set new_custom_cfg($element) [PathRelToAbs $emotion($element)]
+				} else {
+					set new_custom_cfg($element) $emotion($element)
+				}
 			} else {
 				set new_custom_cfg($element) ""
 			}
@@ -983,15 +1077,22 @@ namespace eval ::smiley {
 				::picture::Convert [::skin::GetSkinFile smileys "$new_custom_cfg(file)"] "${destfile}.png"
 				set file "${destfile}.png"
 			}
+
+		    	# Don't forget to delete the temp image...!
+		    	image delete tmp
+
 			if { ![file exists $file] } { set file "" }
 		} else {
 			#We convert animated smiley to animated gif even if we can just load animated gif and save to animated gif
 			#Don't care of extension : as the smiley is animated TkCximage will save to GIF format
 			set filetail_noext [filenoext [file tail "$new_custom_cfg(file)"]]
 			set destfile [file join $HOME smileys $filetail_noext]
-			::picture::Convert [::skin::GetSkinFile smileys "$new_custom_cfg(file)"] "${destfile}.png"
-			set file "${destfile}.png"
-			if { ![file exists $file] } { set file "" }
+			if { [catch {file copy -force [::skin::GetSkinFile smileys "$new_custom_cfg(file)"] "${destfile}.png"}]} {
+				set file ""
+			} else {
+				set file "${destfile}.png"
+				if { ![file exists $file] } { set file "" }
+			}
 		}
 		
 		if { $file == "" } {
@@ -1000,6 +1101,7 @@ namespace eval ::smiley {
 		
 		set emotion(file) $file
 		set emotion(name) $name
+		set emotion(reachable) 1
 		
 		#Create a list of symbols
 		set emotion(text) [list]
@@ -1026,6 +1128,7 @@ namespace eval ::smiley {
 		
 		
 		set emotion(image_name) [image create photo emoticonCustom_std_$emotion(text) -file $emotion(file) -format cximage]
+
 		set custom_emotions($name) [array get emotion]
 
 		#load_smileys
@@ -1048,6 +1151,7 @@ namespace eval ::smiley {
 			raise .new_custom
 			return
 		}
+		set new_custom_cfg(name) $text
 		set new_custom_cfg(text) $text
 		set new_custom_cfg(file) [::skin::GetSkinFile smileys "cache/${file}.png"]
 		newCustomEmoticonGUI
@@ -1107,8 +1211,29 @@ namespace eval ::smiley {
 	return 0
 	
 	}
-}
 
+	proc UnloadEmoticons { } {
+		global custom_emotions
+
+		foreach name [array names custom_emotions] {
+			array set emotion $custom_emotions($name)
+			catch {[image delete $emotion(image_name)]}
+			catch {[image delete $emotion(preview)]}
+		}
+
+		unset custom_emotions
+	}
+	proc PurgeUnreachables { } {
+		global custom_emotions
+		foreach name [array names custom_emotions] {
+			array set emotion [set custom_emotions($name)]
+			if {$emotion(reachable) == 0 } {
+				::smiley::NewCustomEmoticonGUI_Delete $name
+			}
+		}
+	}
+
+}
 
 
 
@@ -1159,7 +1284,7 @@ proc custom_smile_subst { chatid tw {textbegin "0.0"} {end "end"} } {
 			if { [string compare [$tw get $pos $endpos]  $symbol] != 0} {
 				continue
 			} else {
-				for { set i $pos } {$i <= $endpos && !$skip } {set i [$tw index $i+1c]} {
+				for { set i $pos } {$i < $endpos && !$skip } {set i [$tw index $i+1c]} {
 					set tags [$tw tag names $i]
 					foreach tag_name $tags {
 						set elided [$tw tag cget $tag_name -elide]
@@ -1186,13 +1311,15 @@ proc custom_smile_subst { chatid tw {textbegin "0.0"} {end "end"} } {
 			set copyMenu "${tw}.emoticonCustom_std_$file"
 			if { ![winfo exists $copyMenu] } {
 				menu $copyMenu -tearoff 0 -type normal
-				$copyMenu add command -label "[trans emoticon_steal] ($symbol)" -command "::smiley::addSmileyFromTW {$file} {[string map {"\\" "\\\\"} $symbol]}"
+				$copyMenu add command -label "[trans emoticon_steal]-> $symbol" -command "::smiley::addSmileyFromTW [list $file] [list $symbol]"
 				$tw tag bind $twTag <Enter> "$tw configure -cursor hand2"
 				$tw tag bind $twTag <Leave> "$tw configure -cursor xterm"
 				$tw tag bind $twTag <<Button1>> "tk_popup $copyMenu %X %Y"
+				$tw tag bind $twTag <<Button3>> "tk_popup $copyMenu %X %Y"
 		    }
 			
 			set smileyIdx [$tw image create $endpos -image "emoticonCustom_std_$file" -padx 0 -pady 0]
+
 			$tw tag add $twTag $smileyIdx
 			$tw tag remove smiley $endpos
 		    
@@ -1206,33 +1333,27 @@ proc custom_smile_subst { chatid tw {textbegin "0.0"} {end "end"} } {
 
 #Called from the protocol layer to parse a x-mms-emoticon message
 proc parse_x_mms_emoticon { data chatid } {
-    upvar #0 [string map {: _} ${chatid} ]_smileys smile
+	upvar #0 [string map {: _} ${chatid} ]_smileys smile
 
 
-    #Line below changed from != -1 to == 0 because -1 means
-    #"enabled but imagemagick unavailable"
-    if { [::config::getKey getdisppic] == 0 } { return }
+	#Line below changed from != -1 to == 0 because -1 means
+	#"enabled but imagemagick unavailable"
+	if { [::config::getKey getdisppic] == 0 } { return }
 
-    set start 0
-    while { $start < [string length $data]} {
-	set end [string first "	" $data $start]
-	set symbol [encoding convertfrom identity [string range $data $start [expr {$end - 1}]]]
-	set start [expr {$end + 1}]
-	set end [string first "	" $data $start]
-	set msnobj [string range $data $start [expr {$end - 1}]]
-	set start [expr {$end + 1}]
-
-	set smile($symbol) "$msnobj"
-    }
-	
-    status_log "Got smileys : [array names smile]\n" 
+	#data is in format: symbol<TAB>msnobj<TAB> (repeated for every custom emoticon)
+	set emoticonlist [split [string trimright $data] "\t"]
+	foreach {symbol msnobj} $emoticonlist {
+		set symbol [encoding convertfrom identity $symbol]
+		set smile($symbol) "$msnobj"
+	}
 
+	status_log "Got smileys : [array names smile]\n" 
 }
 
 proc process_custom_smileys_SB { txt {animated 0} } {
 	global custom_emotions
 	
-	set msg ""
+	set result [list]
 	
 	set txt2 [string toupper $txt]
 
@@ -1243,21 +1364,22 @@ proc process_custom_smileys_SB { txt {animated 0} } {
 			status_log "process_custom_smileys_SB: Custom smiley $name doesn't exist in custom_emotions array!!\n" red
 			continue
 		}
-		
+
 		array set emotion $custom_emotions($name)
 		foreach symbol [encoding convertto identity $emotion(text)] {
 			set symbol2 [string toupper $symbol]
 		
-			set file $emotion(file)
+			set file [PathRelToAbs $emotion(file)]
 			if { ($animated && ([ info exists emotion(animated) ] && [ is_true $emotion(animated) ])) ||
 			     (!$animated && (! [ info exists emotion(animated) ] || ! [ is_true $emotion(animated) ]))} {
 				if { [info exists emotion(casesensitive)] && [is_true $emotion(casesensitive)] } {
 					if {  [string first $symbol $txt] != -1 } {
-						append msg "$symbol\t[create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]\t"
+						lappend result [list $symbol [create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]]
 					}
 				} else {
 					set msnobj ""
 					set startidx 0
+					set variations [list]
 					while {  [string first $symbol2 $txt2 $startidx] != -1 } {
 						if { $msnobj == "" } {
 							set msnobj [create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]
@@ -1266,14 +1388,19 @@ proc process_custom_smileys_SB { txt {animated 0} } {
 						set idx [string first $symbol2 $txt2 $startidx]
 						set startidx [expr {$idx + [string length $symbol2]}]
 						set symbol [string range $txt $idx [expr {$startidx - 1}]]
-						append msg "$symbol\t$msnobj\t"
+
+						# Avoid adding multiple times the same symbol
+						if { [lsearch $variations $symbol] == -1 } {
+							lappend variations $symbol
+							lappend result [list $symbol $msnobj]
+						}
 					}
 				}
 			}
 		}
 	}
 	
-	return $msg
+	return $result
 }
 
 
